@@ -2,7 +2,9 @@
 ## package 'secr'
 ## D.design.MS.R
 ## Prepare density design matrix
-## last changed 2009 02 15, 2009 06 11, 2009 07 11
+## 
+## force levels=sessionlevels for dframe$session 2010 02 25
+
 ############################################################################################
 ## NOTE does not standardize sessioncov, maskcov
 ############################################################################################
@@ -73,8 +75,9 @@ D.designdata <- function (mask, Dmodel, grps, sessionlevels, sessioncov = NULL)
         dframe$g <- insertdim(factor(grps), 2, dims)
     } 
 
+    ## force levels=sessionlevels 2010 02 25
     if ('session' %in% vars) {
-       dframe$session <- insertdim(factor(sessionlevels), 3, dims)
+       dframe$session <- insertdim(factor(sessionlevels, levels=sessionlevels), 3, dims)
     } 
 
     ## all autovars should have now been dealt with 
