@@ -2,11 +2,12 @@
 ## plot.secr.R
 ## Method for plotting detection function from fitted secr object
 ## 2009 02 12 2009 08 09 2009 09 13 (limits)
+## 2009 11 04 signal strength
 ############################################################################################
 
 plot.secr <- function (x, newdata=NULL, add = FALSE, 
     sigmatick = FALSE, rgr = FALSE, limits = TRUE, alpha = 0.05, xval = 0:200, 
-    ylim = NULL, xlab = 'Distance  (m)', ylab = 'Detection probability', ...) 
+    ylim = NULL, xlab = NULL, ylab = NULL, ...) 
 {
     HN <- function (pars, r) {
         g0 <- pars[1]; sigma <- pars[2]
@@ -36,11 +37,24 @@ plot.secr <- function (x, newdata=NULL, add = FALSE,
     }
     SS <- function (pars, r) {
         beta0 <- pars[1]; beta1 <- pars[2]; sdS <- pars[3]
+        if (is.null(x$details$cutval)) 
+            stop ('require cut parameter for signal strength plot')
         mu <- beta0 + beta1 * r
-        1 - pnorm (q=x$cut, mean=mu, sd=sdS)
+        1 - pnorm (q=x$details$cutval, mean=mu, sd=sdS)
+    }
+    SSS <- function (pars, r) {
+        beta0 <- pars[1]; beta1 <- pars[2]; sdS <- pars[3]
+        if (is.null(x$details$cutval)) 
+            stop ('require cut parameter for signal strength plot')
+        ## spherical so assume distance r measured from 1 m
+        mu <- beta0 - 10 * log ( r^2 ) / 2.302585 + beta1 * (r-1)
+        mu[r<1] <- beta0
+        1 - pnorm (q=x$details$cutval, mean=mu, sd=sdS)
     }
   
-    gline <- function (predicted, rowi = 1) {
+    gline <- function (predicted, rowi = 1, eps = 1e-10) {
+        ## eps is used to limit y to range where gradient() works
+        ## may need later adjustment
         if (!is.data.frame(predicted)) {
             out <- list()
             for (i in 1:length(predicted)) 
@@ -65,15 +79,16 @@ plot.secr <- function (x, newdata=NULL, add = FALSE,
             pars <- predicted[parnames,'estimate']  
       
             pars[is.na(pars)] <- unlist(x$fixed)
-            dfn <- switch (x$detectfn+1, HN, HZ, EX, UN, CHN, WEX,,,,,SS,SS)
+            dfn <- switch (x$detectfn+1, HN, HZ, EX, UN, CHN, WEX,,,,,SS,SSS)   ## omits binary SS!
             if (sigmatick) {
               sigma <- pars[2]
               y <- dfn(pars, sigma)
               dy <- par()$usr[4]/20
               segments (sigma, y-dy, sigma, y+dy)
             }
-      
+
             y <- dfn(pars, xval)
+  
             if (rgr) {
               y <- xval * y
               ymax <- par()$usr[4]
@@ -147,8 +162,9 @@ plot.secr <- function (x, newdata=NULL, add = FALSE,
                 ## lcl <- pmax(y - z*se,0)  # on natural scale
                 ## ucl <- pmin(y + z*se,1)
 
-                lcl <- invlogit (logit(y) - z*se) 
-                ucl <- invlogit (logit(y) + z*se)
+                ## limits on link scale
+                lcl <- ifelse ((y>eps) & (y<(1-eps)), invlogit (logit(y) - z*se), NA) 
+                ucl <- ifelse ((y>eps) & (y<(1-eps)), invlogit (logit(y) + z*se), NA)
 
                 lines (xval, lcl, lty=2, ...)            
                 lines (xval, ucl, lty=2, ...)            
@@ -175,25 +191,34 @@ plot.secr <- function (x, newdata=NULL, add = FALSE,
             }
             if (is.data.frame(temp)) maxg0 <- getmax(temp)
             else maxg0 <- max(sapply (temp, getmax))
+
             if (is.na(maxg0)) maxg0 <- x$fixed$g0
-            if (maxg0 > 0.75) maxg0 <- 1          
+            if (maxg0 > 0.75) maxg0 <- max(maxg0,1)          
             ylim <- c(0, maxg0)
         }
     }
-
-    if (!add) plot (type ='n', 0,0, xlim=range(xval), ylim=ylim, 
-        xlab=xlab, ylab=ylab,...)
-
+    if (!add) {
+        if (is.null(xlab)) 
+            xlab <- 'Distance  (m)'
+        if (is.null(ylab)) {
+           binomN <- ifelse(is.null(x$details$binomN),0,x$details$binomN)
+           dlambda <- (detector(traps(x$capthist)) %in% c('quadratbinary', 'polygon')) |
+               ((detector(traps(x$capthist)) %in% c('count','quadratcount')) & (binomN==0))
+           if (dlambda) 
+               ylab <- 'Detection lambda'
+           else
+               ylab <- 'Detection probability'
+        }      
+        plot (type ='n', 0,0, xlim=range(xval), ylim=ylim, 
+            xlab=xlab, ylab=ylab,...)
+    }
     invisible(gline(temp))
-    
-
 }
 ############################################################################################
 
 detectfnplot <- function (detectfn, pars, details = NULL,
     add = FALSE, sigmatick = FALSE, rgr = FALSE,  
-    xval = 0:200, ylim = NULL, xlab = 'Distance  (m)', 
-    ylab = 'Detection probability', ...) 
+    xval = 0:200, ylim = NULL, xlab = NULL, ylab = NULL, ...) 
 {
     HN <- function (pars, r) {
         g0 <- pars[1]; sigma <- pars[2]
@@ -223,22 +248,26 @@ detectfnplot <- function (detectfn, pars, details = NULL,
     }
     SS <- function (pars, r) {
         beta0 <- pars[1]; beta1 <- pars[2]; sdS <- pars[3]
-        if (is.null(details$spherical) | is.null(details$cut)) 
-            stop ('require spherical and cut parameters for signal strength plot')
-        ## if spherical, assume distance r measured from 1 m
-        if (details$spherical) {
-            mu <- beta0 - 10 * log ( r^2 ) / 2.302585 + beta1 * (r-1)
-            mu[r<1] <- NA
-        }
-        else
-            mu <- beta0 + beta1 * r
-        1 - pnorm (q=details$cut, mean=mu, sd=sdS)
+        if (is.null(details$cutval)) 
+            stop ('require cut parameter for signal strength plot')
+        mu <- beta0 + beta1 * r
+        1 - pnorm (q=details$cutval, mean=mu, sd=sdS)
+    }
+  
+    SSS <- function (pars, r) {
+        beta0 <- pars[1]; beta1 <- pars[2]; sdS <- pars[3]
+        if (is.null(details$cutval)) 
+            stop ('require cut parameter for signal strength plot')
+        ## spherical so assume distance r measured from 1 m
+        mu <- beta0 - 10 * log ( r^2 ) / 2.302585 + beta1 * (r-1)
+        mu[r<1] <- beta0
+        1 - pnorm (q=details$cutval, mean=mu, sd=sdS)
     }
   
     gline <- function (pars) {
         ## here pars is a vector of parameter values
 
-        dfn <- switch (detectfn+1, HN, HZ, EX, UN, CHN, WEX,,,,,SS,SS)
+        dfn <- switch (detectfn+1, HN, HZ, EX, UN, CHN, WEX,,,,,SS,SSS)
         if (sigmatick) {
             sigma <- pars[2]
             y <- dfn(sigma, pars)
@@ -276,10 +305,58 @@ detectfnplot <- function (detectfn, pars, details = NULL,
         }
     }
 
-    if (!add) plot (type ='n', 0,0, xlim=range(xval), ylim=ylim, 
-        xlab=xlab, ylab=ylab,...)
+    if (!add) {
+        if (is.null(xlab)) 
+            xlab <- 'Distance  (m)'
+        if (is.null(ylab))
+            ylab <- 'Detection'
+        plot (type ='n', 0,0, xlim=range(xval), ylim=ylim, 
+            xlab=xlab, ylab=ylab,...)
+    }
 
     invisible( apply (pars, 1, gline) )
 
 }
 ############################################################################################
+
+attenuationplot <- function (pars, add = FALSE, spherical = TRUE,
+    xval = 0:200, ylim = NULL, xlab = NULL, ylab = NULL, ...) {
+
+    mufn <- function (pars, r) {
+        beta0 <- pars[1]
+        beta1 <- pars[2]
+        ## if spherical, assume distance r measured from 1 m
+        if (spherical) {
+            mu <- beta0 - 10 * log ( r^2 ) / 2.302585 + beta1 * (r-1)
+            mu[r<1] <- beta0
+        }
+        else
+            mu <- beta0 + beta1 * r
+        mu
+    }
+
+    aline <- function (pars, r) {
+        y <- mufn (pars, xval)
+        lines (xval, y, ...)
+        data.frame(x=xval, y=y)
+    }
+
+    if (!is.matrix(pars)) pars <- matrix(pars, nr=1)
+
+    if (is.null(ylim)) {
+        lower <- min(apply(pars, 1, mufn, r = max(xval)))
+        ylim <- c(lower, max(pars[,1]))
+    }
+
+    if (!add) {
+        if (is.null(xlab)) 
+            xlab <- 'Distance  (m)'
+        if (is.null(ylab))
+            ylab <- 'Acoustic power (dB)'
+        plot (type ='n', 0,0, xlim=range(xval), ylim=ylim, 
+            xlab=xlab, ylab=ylab,...)
+    }
+    invisible( apply (pars, 1, aline) )
+
+}
+
