@@ -1,9 +1,11 @@
 ############################################################################################
 ## package 'secr'
 ## secr.design.MS.R
-## last changed 2009 02 13, 2009 06 23, 2009 07 13, 2009 08 20 disable bk, Bk
+## 2009 08 20 disable bk, Bk
 ## 2009 10 08 sighting
-## 2009 12 22 fix bug in 'session'
+## 2009 12 10 mixtures
+## 2009 12 13 mixtures pmix ~ h2
+
 ############################################################################################
 ## source ('d:\\density secr 1.2\\secr\\R\\secr.design.MS.R')
 ############################################################################################
@@ -75,6 +77,7 @@ secr.design.MS <- function (capthist, models, timecov = NULL, sessioncov = NULL,
     parnames <- names(models)                 # typically c('g0', 'sigma', 'z')
     vars     <- unique (unlist(sapply (models, all.vars)))
     vars     <- vars[!(vars %in% groups)]     # groups treated separately
+    nmix     <- get.nmix(models)
 
 ##    if (sum(c('b','B','bk','Bk') %in% vars) > 1) 
     if (sum(c('b','B') %in% vars) > 1) 
@@ -91,18 +94,23 @@ secr.design.MS <- function (capthist, models, timecov = NULL, sessioncov = NULL,
     MS   <- inherits(capthist, 'list') # logical for multi-session
     sessionlevels <- session(capthist)
     if (is.null(sessionlevels)) sessionlevels <- '1'
+    getnk <- function (object) {
+        ifelse (detector(object) %in% c('polygon','transect'), 
+            length(table(polyID(object))), 
+            nrow(object))
+    }
 
     if (MS) {
         R <- length(capthist)
         n <- ifelse (bygroup, ngrp, max(sapply(capthist, nrow))) # max over sessions 
         S <- max(sapply(capthist, ncol))                         # max over sessions
-        K <- max(sapply(trps, nrow))                             # max over sessions
+        K <- max(sapply(trps, getnk))                            # max over sessions
     }
     else {
         R <- 1
         n <- ifelse (bygroup, ngrp, nrow(capthist))
         S <- ncol(capthist)
-        K <- nrow(trps)
+        K <- getnk(trps)
     }
 
     ## timecov may be a vector or a dataframe or a list of vectors or a list of data frames
@@ -120,18 +128,17 @@ secr.design.MS <- function (capthist, models, timecov = NULL, sessioncov = NULL,
         if (nrow(sessioncov) != R) stop('wrong number of session covariates')
     }
 
-    dims <- c(R,n,S,K)  # 'virtual' dimensions
+    dims <- c(R,n,S,K,nmix)  # 'virtual' dimensions
+    autovars <- c('session','g','t','T','b','B','kcov','tcov', 'sighting', 'h2', 'h3')
+    if (is.null(dframe)) dframe <- data.frame(dummy=rep(1, R*n*S*K*nmix))
 
-##    autovars <- c('session','g','t','b','B','bk','Bk','kcov','tcov')
-    autovars <- c('session','g','t','b','B','kcov','tcov', 'sighting')
-    if (is.null(dframe)) dframe <- data.frame(dummy=rep(1, R*n*S*K))
-
-##    dframe$session <- factor( insertdim (sessionlevels, 1, dims) )
-## fixed bug 2009 12 22 MGE
+## BUG FIXED 2009 12 22
+## Default action of following was to sort levels
+## but these are allowed to be in arbitrary order
+##    dframe$session <- factor( insertdim (sessionlevels, 1, dims))
 
     dframe$session <- factor( insertdim (sessionlevels, 1, dims), levels = sessionlevels)
 
-
     #--------------------------------------------------------------------------
     if ('t' %in% vars) 
         dframe$t <- factor( insertdim (1:S, 3, dims) )
@@ -173,6 +180,7 @@ secr.design.MS <- function (capthist, models, timecov = NULL, sessioncov = NULL,
                 if (inherits(capthist, 'list')) levels(zcov[[1]][,facname]) 
                 else levels(zcov[,facname])
             } 
+
             ## list with levels for each grouping factor named in 'groups'
             grouplist <- lapply(groups, MSlevels)
             grouping <- expand.grid(grouplist) ## one column per group
@@ -194,7 +202,7 @@ secr.design.MS <- function (capthist, models, timecov = NULL, sessioncov = NULL,
 
     ## general for behavioural response fields
 
-    prox <- detector(trps)[[1]] == 'proximity' ## assume all sessions the same  
+    prox <- detector(trps)[[1]] %in% c('proximity', 'count', 'quadratbinary', 'quadratcount', 'polygon') ## assume all sessions same type 
     makeb <- function (caphist) {      ## global response
         if (prox) {
            temp0 <- apply(caphist, 1:2, sum)
@@ -282,13 +290,27 @@ secr.design.MS <- function (capthist, models, timecov = NULL, sessioncov = NULL,
     #--------------------------------------------------------------------------
 
     if ('tcov' %in% vars) {
-        if (is.null(timecov) | length(timecov) != S) 
+        if (is.null(timecov)) 
             stop ('Require valid time covariate timecov')
-        if (is.data.frame(timecov)) timecov <- timecov[,1,drop=F]  ## retain only first
-        else timecov <- lapply (timecov, function(x) pad1(x[,1], S))         
+        if (length(unlist(timecov)) > S)                                             ### CHECK
+            warning ('length of time covariate exceeds number of occasions')
+        if (is.data.frame(timecov)) {
+            if (nrow(timecov) < S)
+                stop ('Require valid time covariate timecov')
+            timecov <- timecov[,1,drop=F]  ## retain only first
+        }
+        else {
+            if (any(sapply(timecov, nrow) < S))
+                stop ('Require valid time covariate timecov')
+            timecov <- lapply (timecov, function(x) pad1(x[,1], S))         
+        }
         dframe$tcov <- insertdim (unlist(timecov), c(3,1), dims) 
     }
 
+    if ('T' %in% vars) {
+        dframe$T <- insertdim (0:(S-1), c(3,1), dims) 
+    }
+
     #--------------------------------------------------------------------------
     if ('kcov' %in% vars) {
         if (is.null(trapcov))
@@ -297,7 +319,17 @@ secr.design.MS <- function (capthist, models, timecov = NULL, sessioncov = NULL,
         else trapcov <- lapply (trapcov, function(x) pad1(x[,1], K)) 
         dframe$kcov <- insertdim(unlist(trapcov), c(4,1), dims)
     }
-    
+
+    #--------------------------------------------------------------------------
+
+
+    ## h2 or h3 
+    if (nmix > 1) {
+        mixture <- paste('h',nmix,sep='')
+        dframe[,mixture] <- insertdim(factor(1:nmix), 5, dims)
+    }
+
+    #--------------------------------------------------------------------------   
     ## all autovars should have now been dealt with 
     vars <- vars[!vars %in% autovars]
 
@@ -321,10 +353,12 @@ secr.design.MS <- function (capthist, models, timecov = NULL, sessioncov = NULL,
     make.designmatrix <- function (formula, prefix, ...) {
      # combine formula and dframe to generate design matrix
         if (is.null(formula)) {
-            list (model = NULL, index = rep(1,R*n*S*K))
+            list (model = NULL, index = rep(1,R*n*S*K*nmix))
         }
         else {
             tempmat <- model.matrix(formula, dframe, ...)
+            ## drop pmix beta0 column from design matrix
+            if (prefix=='pmix') tempmat <- tempmat[,-1,drop=FALSE]      
             temp <- make.lookup (tempmat)   # retain unique rows
             list (model=temp$lookup, index=temp$index)
         } 
@@ -336,9 +370,13 @@ secr.design.MS <- function (capthist, models, timecov = NULL, sessioncov = NULL,
 
     # list with one component per real parameter
     # each of these is a list with components 'model' and 'index'
-    designMatrices <- lapply (models, make.designmatrix)
 
-    # dim(indices) = c(R*n*S*K, npar)
+##    designMatrices <- lapply (models, make.designmatrix)
+    designMatrices <- sapply (1:length(models), simplify=FALSE, 
+        function (x) make.designmatrix(models[[x]], names(models[x])))
+    names(designMatrices) <- names(models)
+
+    ## dim(indices) = c(R*n*S*K*nmix, npar)
     indices <- sapply (designMatrices, function(x) x$index)
 
     indices <- matrix(unlist(indices), nc=npar)
@@ -356,12 +394,12 @@ secr.design.MS <- function (capthist, models, timecov = NULL, sessioncov = NULL,
 
     #--------------------------------------------------------------------
     # PIA = Parameter Index Array
-    #       index to row of parameterTable for a given R,n,S,K
+    #       index to row of parameterTable for a given R,n,S,K,nmix
     # dim(parameterTable) = c(uniqueparcomb, npar)
     #       index to row of designMatrix for each real parameter
     #--------------------------------------------------------------------
 
-    PIA <- array(indices2$index, dim=c(R,n,S,K))
+    PIA <- array(indices2$index, dim=c(R,n,S,K,nmix))
 
     parameterTable <- indices2$lookup
 
@@ -373,18 +411,24 @@ secr.design.MS <- function (capthist, models, timecov = NULL, sessioncov = NULL,
     #--------------------------------------------------------------------
 
     # 'used' is list if MS
+
+## OOPS - TOO TIRED TO THINK THROUGH THE IMPLEMENTATION OF USE WITH MIXTURES 
+## UNSURE FROM HERE 2009 12 10
+
     if ((!is.null(used)) & (length(used)>0)) {
         allused <- unlist(used)
         if (!is.null(allused))
         if (any(!allused)) {
             if (!MS) {
-                PIA[1, , ,] <- PIA[1, , ,] * rep(t(used),rep(n,S*K))
+##                PIA[1, , ,] <- PIA[1, , ,] * rep(t(used),rep(n,S*K))
+                PIA[1, , , ,] <- PIA[1, , , ,] * rep(rep(t(used),rep(n,S*K)),nmix)
             }
             else for (r in 1:R) {
                 use <- array (0, dim=c(S,K))
                 temp <- t(used[[r]])  # dim (S',K')
                 use[1:nrow(temp), 1:ncol(temp)] <- temp  # padding
-                PIA[r, , ,] <- PIA[r, , ,] * rep(use,rep(n,S*K))
+##                PIA[r, , , ] <- PIA[r, , , ] * rep(use,rep(n,S*K))
+                PIA[r, , , ,] <- PIA[r, , , ,] * rep(rep(use,rep(n,S*K)),nmix)
             } 
         }
     }
